

--######################################################################################--
-- Addon Data & Setting Defaults
--######################################################################################--

-- root namespace so collisions aren't a problem
local Cabalist = {}

-- addon data							
Cabalist.lastUpdateTime = Inspect.System.Time()
Cabalist.updateInterval = .2
Cabalist.buffsChanged = true
Cabalist.context = UI.CreateContext("Cabalist")  	-- set us up the ux

-- common settings
Cabalist.settings = {}								-- should set this to a default here TODO later
Cabalist.settings.version = 1.0						-- settings contract version, this should change when setting data becomes incompatible
Cabalist.settings.debug = false						-- refresh rate in seconds

-- buffs frame settings
Cabalist.settings["icons"] = {}
Cabalist.settings["icons"].x = 1000					-- assumes 1600x1200 res, probably a bad assumption, but w/e
Cabalist.settings["icons"].y = 400


--######################################################################################--
-- Buff data tables (this may change between game updates)
--######################################################################################--

-- ideally want to have buff ID as the value, but can use buff texture.

-- Sigils
Cabalist.sigil = { 
	["Sigil of Ravaging"] 	=	1, 	
	["Sigil of Secrecy"]	=	1,	
	["Sigil of Binding"]	=	1,		
	["Sigil of Power"]		=	1,	
	["Sigil of Ruin"]		=	1,	
	["Sigil of Woe"]		=	1,			
}

-- Special (sign of asias)
Cabalist.special = {
	["Sign of Asias"] 		= 	1,				
}

-- Decay 
Cabalist.decay = {
	["Lurking Decay"]		=	1,
}


--######################################################################################--
-- Cabalist Buff Watch
--######################################################################################--

-- generic function to create the icon style I like
function Cabalist.createIcon (context, r, g, b, a)

	-- create the base icon
	buffIcon = UI.CreateFrame("Texture", "Icon", context)
	buffIcon:SetHeight(36)
	buffIcon:SetWidth(36)
	buffIcon:SetBackgroundColor(r, g, b, a)		
	buffIcon:SetLayer(1)
	
	-- create icon texture, center it over the icon and make it smaller
	buffIcon.icon = UI.CreateFrame("Texture", "IconTexture", buffIcon)	
	buffIcon.icon:SetPoint("CENTER", buffIcon, "CENTER")
	buffIcon.icon:SetHeight(32)
	buffIcon.icon:SetWidth(32)
	buffIcon.icon:SetLayer(2)
	
	-- create icon text
	buffIcon.text = UI.CreateFrame("Text", "IconText", buffIcon)
	buffIcon.text:SetFontSize(28)
	buffIcon.text:SetFontColor(1,1,0,1)
	buffIcon.text:SetPoint("CENTER", buffIcon, "CENTER")
	buffIcon.text:SetLayer(3)

	-- hide the icon by default
	buffIcon:SetVisible(false)
	
	-- function to set text, resize, and re-anchor
	function buffIcon:Label(text)
		if text then self.text:SetText(tostring(text)) else self.text:SetText("") end
		self.text:ResizeToText()
		self.text:SetPoint("CENTER", self, "CENTER")
	end
	
	-- return the icon
	return buffIcon
end

-- Create the static icons
Cabalist.sigilIcon = Cabalist.createIcon(Cabalist.context, .5, 0, .7, 1)
Cabalist.decayIcon = Cabalist.createIcon(Cabalist.context, .5, 0, .7, 1)
Cabalist.specialIcon = Cabalist.createIcon(Cabalist.context, .5, 0, .7, 1)

-- This is a function so they can be re-anchored after changing settings
function Cabalist.anchorIcons()
	Cabalist.specialIcon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Cabalist.settings["icons"].x, Cabalist.settings["icons"].y)
	Cabalist.decayIcon:SetPoint("TOPLEFT", Cabalist.specialIcon, "TOPRIGHT", 2, 0)
	Cabalist.sigilIcon:SetPoint("TOPLEFT", Cabalist.decayIcon, "TOPRIGHT", 2, 0)
end

-- refreshes the buffs indicator
function Cabalist.refreshIcons(unit, buffs)

	local buffList = Inspect.Buff.List("player")	
	if not buffList then return end 
	local buffDetails = Inspect.Buff.Detail("player", buffList)
	
	-- hide the frames by default
	Cabalist.sigilIcon:SetVisible(false)
	Cabalist.decayIcon:SetVisible(false)
	Cabalist.specialIcon:SetVisible(false)
	
	-- go through current buffs
	-- look for a match and un-hide & set the icon as-appropriate
	for id, buff in pairs(buffDetails) do

		-- check for sigils
		-- can only ever be one sigil
		if (Cabalist.sigil[buff.name] ~= nil) then
			Cabalist.sigilIcon.icon:SetTexture("Rift", buff.icon)
			Cabalist.sigilIcon:SetVisible(true)
		end
		
		-- check for decay
		-- I can special case this I suppose, but w/e
		if (Cabalist.decay[buff.name] ~= nil) then
			Cabalist.decayIcon.icon:SetTexture("Rift", buff.icon)
			
			-- set decay stacks
			if (buff.stack == nil) then
				Cabalist.decayIcon:Label("I")			
			elseif (buff.stack == 2) then
				Cabalist.decayIcon:Label("II")
			else
				Cabalist.decayIcon:Label("III")
			end

			Cabalist.decayIcon:SetVisible(true)
		end
		
		-- check for special (just sign of asias)
		if (Cabalist.special[buff.name] ~= nil) then
			Cabalist.specialIcon.icon:SetTexture("Rift", buff.icon)
			Cabalist.specialIcon:SetVisible(true)
		end
	end
end

-- sets flag that buffs have changed and to redraw on next pulse
function Cabalist.buffCheck(unit, buffs)
	-- unit is truth, not specifier, always convert intended specifiers to units
	if unit ~= Inspect.Unit.Lookup("player") then return end
	Cabalist.buffsChanged = true
end
-- add to buff events (nothing we are interested in uses Change)
table.insert(Event.Buff.Add, {Cabalist.buffCheck, "Cabalist", "Buff Check"})
table.insert(Event.Buff.Change, {Cabalist.buffCheck, "Cabalist", "Buff Check"})
table.insert(Event.Buff.Remove, {Cabalist.buffCheck, "Cabalist", "Buff Check"})


--######################################################################################--
-- Main Pulse Function																	--
--######################################################################################--

-- function that does the buff checking.
-- is throttled by both time & event triggers
function Cabalist.refresh()
	-- throttle our calls to the updateInterval so we dont' eat up shit tons of cpu.
	local currentTime = Inspect.System.Time()
	if (currentTime - Cabalist.lastUpdateTime < Cabalist.updateInterval) then return end
	Cabalist.lastUpdateTime = currentTime
	
	-- Buff Watch
	if (Cabalist.buffsChanged) then
		Cabalist.refreshIcons()
		Cabalist.buffsChanged = false
	end
end
-- register event for updating
table.insert(Event.System.Update.Begin, {Cabalist.refresh, "Cabalist", "Periodic Update"})


--######################################################################################--
-- Data Save & Load																		--
--######################################################################################--

-- saves state to saved variables
function Cabalist.saveState(addon)
	-- saved variables returns addon
	if (addon ~= "Cabalist") then return end
	
	if (Cabalist.settings.debug) then print("DEBUG: Saving Cabalist State") end
	
	-- save settings
	Cabalist_settings = Cabalist.settings
	
	if (Cabalist.settings.debug) then print("DEBUG: Cabalist State Saved") end
end

-- loads state from saved variables
function Cabalist.loadState(addon)
	-- saved variables returns addon
	if (addon ~= "Cabalist") then return end
	
	if (Cabalist.settings.debug) then print("DEBUG: Loading Cabalist State") end
	
	-- if the contract changes, settings go to default
	if (Cabalist_settings and Cabalist_settings.version and Cabalist_settings.version == 1.0) then
		Cabalist.settings = Cabalist_settings
	end
	
	-- update the anchor points 
	Cabalist.anchorIcons()
	
	if (Cabalist.settings.debug) then print("DEBUG: Cabalist State Loaded") end
end

-- Save & Load state events
table.insert(Event.Addon.SavedVariables.Save.Begin, {Cabalist.saveState, "Cabalist", "Save state"})
table.insert(Event.Addon.SavedVariables.Load.End, {Cabalist.loadState, "Cabalist", "Load state"})



--######################################################################################--
-- Command line options and utilities													--
--######################################################################################--

-- strips off the first word of the command line, leaves the rest for future parsing
-- probably useful in other mods, this is...consider for library project
local function nextToken (str)
	local first, rest
 	_, _, first, rest = string.find(str, "^(%w+)%s?(.*)$")
	return first, rest
end

-- Print usage
function Cabalist.opt_help ()
	print("Cabalist Usage:")
	print("\tmove <x> <y>\tMove frame anchor to (x,y) from topleft.")
	print("\tdebug\t\t\tToggle debug mode.")
	print("\thelp\t\t\t\tThis usage message.")
end

-- move around the Cabalist frame
function Cabalist.opt_move(pos)
	-- validate input data and update appropriate frame
	local x, y = pos:match("^(%d+)%s+(%d+)$")
	x, y = tonumber(x), tonumber(y)
	if (x and y) then
		Cabalist.settings["icons"].x = x
		Cabalist.settings["icons"].y = y
		print(string.format("Cabalist %s frame moved to %d, %d", "icons", Cabalist.settings["icons"].x, Cabalist.settings["icons"].y))
	else
		print("Move requires two positive integers as input.")
	end
	-- refresh anchor positions
	Cabalist.anchorIcons()
end

-- Toggle debug
function Cabalist.opt_debug()
	if (Cabalist.settings.debug) then
		Cabalist.settings.debug = false
		print("Cabalist Debug Disabled.")
	else
		Cabalist.settings.debug = true
		print("Cabalist Debug Enabled.")
	end
end

-- root command parser
-- be nice to have a library for this shit, put on to-do list
function Cabalist.cmd_Cabalist (args)
	local opt, args = nextToken(args)
	if (opt == "debug") then
		Cabalist.opt_debug()
	elseif (opt == "move") then
		Cabalist.opt_move(args)
	else
		Cabalist.opt_help()
	end
end

-- slash command registration, use full name and the short "cab" for the lazy bastards like myself
table.insert(Command.Slash.Register("cabalist"),{Cabalist.cmd_Cabalist, "Cabalist", "Slash command"})
table.insert(Command.Slash.Register("cab"),{Cabalist.cmd_Cabalist, "Cabalist", "Slash command"})





